"""tmux session manager.

:copyright: Copyright 2013- Tony Narlock.
:license: MIT, see LICENSE for details
"""

from __future__ import annotations

import logging

from . import cli, util
from .__about__ import (
    __author__,
    __copyright__,
    __description__,
    __email__,
    __license__,
    __package_name__,
    __title__,
    __version__,
)

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())
