"""Constant variables for tmuxp workspace functionality."""

from __future__ import annotations

import logging

logger = logging.getLogger(__name__)

VALID_WORKSPACE_DIR_FILE_EXTENSIONS = [".yaml", ".yml", ".json"]
